/*
 * This program is intended to be run (as root) as part of a pipeline;
 * it will send a sighup to named every time it receives input, from another
 * program -- which presumably cannot be trusted to run as root.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>

static input_buffer_size = 4095;

int named_pid () {
	char *filename = "/var/run/named.pid";
	char input_buffer[input_buffer_size + 1];
	int fd;
	int bytes_read;
	int pid;
	if (-1 == (fd = open (filename, O_RDONLY))) {
		fprintf (stderr, "%s: ", filename);
		perror ("open");
		return 0;
	}
	
	bytes_read = read(fd, input_buffer, input_buffer_size);
	if (-1 == bytes_read) {
		fprintf (stderr, "%s: ", filename);
		perror ("read");
		goto fail;
	}
	if (0 == bytes_read) {
		fprintf (stderr, "%s: %s\n", filename, 
			"end-of-file at beginning of file");
		goto fail;
	}

	input_buffer[bytes_read] = '\0';	

	pid = atoi(input_buffer);  /* error checking is unimportant now */
	close (fd);
	return pid;

    fail:
	close (fd);
	return 0;
}

/* main(){printf("%d\n", named_pid());} */

int main(int argc, char **argv) {
	char input_buffer[input_buffer_size + 1];
	char bytes_read;
	int pid;
	
	while (1) {
		bytes_read = read (0, input_buffer, input_buffer_size);
		if (-1 == bytes_read) {
			fprintf (stderr, "%s: ", "standard input");
			perror ("read");
			exit (-1);
		}
		if (0 == bytes_read) {
			exit (0);
		}
		pid = named_pid();
		
		if (-1 == kill (pid, SIGHUP)) {
			fprintf (stderr, "pid %d: ", pid);
			perror ("kill");
		}
	}
}
	
